/*****************************************************************
 * File Name:	uuidlib.c
 * Description:
 * This module contains UDFs to generate and manipulate uuids.
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
*****************************************************************/

#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#include "uuid/uuid.h"

#include "uuidlib.h"

/*****************************************************************
 * fn_guid_create
 * returns the created uuid as a 36 char guid
 ****************************************************************/
char* fn_guid_create(char* guid)
{
	uuid_t uuid;
	uuid_generate(uuid);
	uuid_unparse(uuid, guid);
	return guid;
}

/*****************************************************************
 * fn_guid_create
 * returns the created uuid as a 36 char guid (uppercase)
 ****************************************************************/
char* fn_guid_create2(char* guid)
{
	uuid_t uuid;
	int i;
	uuid_generate(uuid);
	uuid_unparse(uuid, guid);
	for (i = 0; i < 36; i++) {
    guid[i] = toupper(guid[i]);
  }
	return guid;
}

void fn_sysdate(ISC_TIMESTAMP* rc)
{
  time_t timep;
  struct tm times;

  time(&timep);
  gmtime_r(&timep, &times);

  // Inline encode_date
  const int day = times.tm_mday;
  int month = times.tm_mon + 1;
  int year = times.tm_year + 1900;
  if (month > 2)
    month -= 3;
  else {
    month += 9;
    year -= 1;
  }
  const int c = year / 100;
  const int ya = year - 100 * c;
  rc->timestamp_date = (ISC_DATE) (((SINT64) 146097 * c) / 4 +
    (1461 * ya) / 4 + (153 * month + 2) / 5 + day + 1721119 - 2400001);


  rc->timestamp_time = ((times.tm_hour * 60 + times.tm_min) * 60 +
    times.tm_sec) * ISC_TIME_SECONDS_PRECISION;
}

